package net.sf.sido.spring.mvc;

import net.sf.sido.DataFactory;
import net.sf.sido.DataObject;
import net.sf.sido.DataType;
import net.sf.sido.io.DataObjectReader;
import net.sf.sido.io.DataObjectWriter;
import net.sf.sido.pojo.DOFactory;
import net.sf.sido.pojo.DOObject;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.reflect.MethodUtils;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.HttpOutputMessage;
import org.springframework.http.MediaType;
import org.springframework.http.converter.AbstractHttpMessageConverter;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.http.converter.HttpMessageNotWritableException;

import java.io.IOException;

public class AbstractDataObjectHttpMessageConverter<T, N> extends AbstractHttpMessageConverter<T> {

    private final DataFactory factory;
    private final DataObjectReader<?> doReader;
    private final DataObjectWriter<?, N> doWriter;

    public AbstractDataObjectHttpMessageConverter(MediaType mediaType, DataFactory factory, DataObjectReader<?> doReader, DataObjectWriter<?, N> doWriter) {
        super(mediaType);
        this.factory = factory;
        this.doReader = doReader;
        this.doWriter = doWriter;
    }

    @Override
    protected T readInternal(Class<? extends T> clazz, HttpInputMessage inputMessage) throws IOException, HttpMessageNotReadableException {
        DataType expectedType = getExpectedType(clazz, inputMessage);
        DataObject o = doReader.read(inputMessage.getBody(), factory, expectedType);
        if (DataObject.class.isAssignableFrom(clazz)) {
            return (T) o;
        } else if (DOObject.class.isAssignableFrom(clazz)) {
            return (T) DOFactory.getContextFactory().create(o);
        } else {
            throw new IllegalStateException(String.format("Cannot convert DataObject to %s", clazz.getName()));
        }
    }

    private DataType getExpectedType(Class<? extends T> clazz, HttpInputMessage inputMessage) throws IOException {
        String typeName;
        if (DataObject.class.isAssignableFrom(clazz)) {
            typeName = inputMessage.getHeaders().getFirst("sido:type");
        } else if (DOObject.class.isAssignableFrom(clazz)) {
            try {
                typeName = (String) MethodUtils.invokeStaticMethod(clazz, "getQualifiedTypeName", new Object[0]);
            } catch (Exception e) {
                throw new IOException(
                        String.format("Cannot get the type name from %s", clazz),
                        e
                );
            }
        } else {
            throw new IllegalStateException(String.format("Cannot convert DataObject to %s", clazz.getName()));
        }
        if (StringUtils.isNotBlank(typeName)) {
            return factory.getMandatoryType(typeName);
        } else {
            return null;
        }
    }

    @Override
    protected boolean supports(Class<?> clazz) {
        return DataObject.class.isAssignableFrom(clazz) || DOObject.class.isAssignableFrom(clazz);
    }

    @Override
    protected void writeInternal(T t, HttpOutputMessage outputMessage) throws IOException, HttpMessageNotWritableException {
        DataObject o;
        if (t instanceof DataObject) {
            o = (DataObject) t;
        } else if (t instanceof DOObject) {
            o = ((DOObject) t).getDataObject();
        } else {
            throw new IllegalStateException(String.format(
                    "Cannot get DataObject from %s",
                    t.getClass().getName()));
        }
        doWriter.write(outputMessage.getBody(), o);
    }

}
